/* ==============================================================================================================
 * 
 *                            VOIR_DEMAIN_ANIM - Animation des leds, et mode simulation 
 * 
 * =============================================================================================================*/

/* --------------------------------------------------------------------------------------------------------------
 *  mathRandomInt() : rend une valeur aléatoire entre min et max
 *  ------------------------------------------------------------------------------------------------------------- */
int mathRandomInt(int min, int max) {
  
    if (min > max) {  // Swap min and max to ensure min is smaller.
        int temp = min;
        min = max;
        max = temp;
    }
    return min + (rand() % (max - min + 1));
    
}

/* --------------------------------------------------------------------------------------------------------------
 *  neopixel_showAllLed() : traitement simultané de toutes les leds du ruban
 * ------------------------------------------------------------------------------------------------------------- */
void neopixel_showAllLed(Adafruit_NeoPixel *neoPx, uint8_t ledCount, uint8_t r, uint8_t g, uint8_t b) {
  
    int i;
    for (i=0; i < ledCount; i++) {
        neoPx->setPixelColor(i, neoPx->Color(r, g, b));
    }
    neoPx->show();
    
}

/* --------------------------------------------------------------------------------------------------------------
 *  setWeatherNeedle() : positionnement de l'aiguille sur la bonne icône météo.
 *  ------------------------------------------------------------------------------------------------------------- */
void setWeatherNeedle(int pServoPos) {

    if (pServoPos != GL_servoPosition) {            // On n'agit que si besoin de changer de position.
        GL_weatherServo.attach(PIN_SERVOMOTEUR);
        delay(50);
        GL_weatherServo.write(pServoPos);
        delay(250);                                 // il faut laisser au servo le temps de se positionner..
        GL_weatherServo.detach();
        GL_servoPosition = pServoPos;
    }
    
}

/* --------------------------------------------------------------------------------------------------------------
 *  setWeather...() : positionnement du mode météo (Orage, Soleil, Nuage, Soleil voilé)
 *  - Ces fonctions vont positionner l'aiguille du servo, et activer les leds du ruban, en fonction de la météo.
 *  ------------------------------------------------------------------------------------------------------------- */
 
void setWeatherSun(bool pInit) {

    /*
     * En mode initialisation, arrivée rapide du soleil.
     * et sinon, rien à faire, le soleil est toujours là !
     */
     
    if (pInit) {
        setWeatherNeedle(GL_SERVO_SUN) ;
        for (int i = 55; i <= 255; i += 10) {
            neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, i, i, 0);
            delay(15);
        }
    }
         
}

// ----------------
bool GL_Sunny ;
void setWeatherCloudySun(bool pInit) {
  
    /*
     * En mode initialisation, arrivée lente du soleil, qui sera moins jaune
     */

    int i ;
    if (pInit) {
        setWeatherNeedle(GL_SERVO_CLOUDYSUN) ;
        for (i = 55; i <= 255; i += 10) {
            neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, i, i, 55);
            delay(60);
        }
        GL_Sunny = true ;
    }
    
    /*
     * et sinon, alternance soleil / nuages, en douceur ...
     */
     
    else {
        if (GL_Sunny) {
            for (i = 255; i >= 55; i -= 10) {
                neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, i, i, 55);
                delay(60);
            }
        }
        else {
            for (i = 55; i <= 255; i += 10) {
                neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, i, i, 55);
              delay(46);
            }
        }
        GL_Sunny = !GL_Sunny ;
    }
    
}

// ----------------
bool GL_Whiter ;
void setWeatherClouds(bool pInit) {

    /*
     * En mode initialisation, montée vers éclairage tamisé, gris-blanc
     */

    int i ;
    if (pInit) {
        setWeatherNeedle(GL_SERVO_CLOUDS) ;
        for (i = 55; i <= 225; i += 10) {
            neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, i, i, i);
            delay(30);
        }
        GL_Whiter = true ;
    }
    
    /*
     * et sinon, alternance gris plus ou moins clair, en douceur ...
     */
     
    else {
        if (GL_Whiter) {
            for (i = 255; i >= 55; i -= 10) {
                neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, i, i, i);
                delay(40);
            }
        }
        else {
            for (i = 55; i <= 225; i += 10) {
                neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, i, i, i);
                delay(30);
            }
        }
        GL_Whiter = !GL_Whiter ;
    }

}

// ----------------
void setWeatherRain(bool pInit) {

    /*
     * Quelque soit le mode, léger clignotement (reflet pluie, grisé)
     */

    int i ;
    if (pInit) {
        setWeatherNeedle(GL_SERVO_RAIN) ;
    }

    for (i = 0; i < 40; i++) {
         neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, 235, 235, 235);
         delay(20);
         neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, 55, 55, 55);
         delay(10);
    }

}

// ----------------
void setWeatherStorm(bool pInit) {
  
    /*
     * Quelque soit le mode, beaux éclairs.
     */

    int i ;
    if (pInit) {
        setWeatherNeedle(GL_SERVO_STORM) ;
    }

    for (i = 0; i < 20; i++) {
        int b = mathRandomInt(1, 255);
        neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, b, b, b);
        delay(mathRandomInt(1, 80));
    }
    neopixel_showAllLed(&GL_weatherLeds, GL_NB_LEDS, 51, 51, 51);

}


/* --------------------------------------------------------------------------------------------------------------
 *  checkSwitch() : vérifie l'état de l'interrupteur (simu vs réel)
 *  valeur rendue : false = mode simulation, true = mode reel.
 *  ------------------------------------------------------------------------------------------------------------- */
bool checkSwitch() {

    if (digitalRead(PIN_SIMU_SWITCH) == LOW) {
        return(false) ;
    } else {
        return(true) ;
    }
    
}

/* --------------------------------------------------------------------------------------------------------------
 *  checkPotCode() : vérifie l'état du potentiomètre et rend une valeur entre 0 et 4
 *  ------------------------------------------------------------------------------------------------------------- */
int checkPotCode() {
  
      int myPotVal = analogRead(PIN_POTENTIOMETER) ;
      int myPotCode = map(myPotVal, 0, 1024, 0, 4);    
      return(myPotCode) ;

}

/* --------------------------------------------------------------------------------------------------------------
 *  showWeather() : affichage de la météo
 *  - pInit = vrai : on relance une nouvelle séquence météo, faux : on continue sur la même météo
 *  ------------------------------------------------------------------------------------------------------------- */
void showWeather(bool pInit) {

    int myWeatherCode ;
    if (GL_realMode) {                                          // Mode réel ...
        myWeatherCode = GL_weatherAbstract[GL_potCode+1] ;      // ... on va chercher la météo à J+1, J+2, ... J+5 en fonction du potentiomètre.
    } 
    else {                                                      // Mode simulé ...
        myWeatherCode = GL_potCode ;                            // ... le potentiomètre donne directement le jour à simuler.
    }
      
     switch (myWeatherCode) {
        case 0: setWeatherSun(pInit); break;           // Soleil.
        case 1: setWeatherCloudySun(pInit); break;     // Nuages+Soleil.
        case 2: setWeatherClouds(pInit); break;        // Nuages.
        case 3: setWeatherRain(pInit); break;          // Pluie.
        case 4: setWeatherStorm(pInit); break;         // Orages.       
    }

}
